from unittest import TestCase

from tc_hivemind_backend.db.pg_db_utils import setup_db
from tc_hivemind_backend.db.postgresql import PostgresSingleton
from tc_hivemind_backend.db.utils.delete_data import delete_data


class TestDeletePGData(TestCase):
    def setUp(self):
        self.dbname = "test_db"
        setup_db(
            community_id="community123",
            dbname=self.dbname,
            latest_date_query="SELECT 1;",
        )

    def test_delete_no_data(self):
        """
        running a dummy query
        """
        delete_data(deletion_query="SELECT 1;", dbname=self.dbname)

    def test_delete_some_data(self):
        postgres = PostgresSingleton(dbname=self.dbname)
        connection = postgres.get_connection()
        connection.autocommit = True
        with connection.cursor() as cursor:
            cursor.execute(
                """
                DROP TABLE IF EXISTS temp_table;
                """
            )
            cursor.execute(
                """
                CREATE TABLE temp_table (
                    id SERIAL PRIMARY KEY,
                    name VARCHAR(50),
                    age INT
                );
            """
            )

            cursor.execute(
                """
                INSERT INTO temp_table (name, age) VALUES
                    ('John Doe', 25),
                    ('Jane Smith', 30),
                    ('Bob Johnson', 22);
            """
            )
        postgres.close_connection()

        delete_data(deletion_query="DELETE FROM temp_table;", dbname=self.dbname)

        postgres = PostgresSingleton(dbname=self.dbname)
        connection = postgres.get_connection()
        with connection.cursor() as cursor:
            cursor.execute("SELECT * FROM temp_table;")
            res = cursor.fetchone()

        postgres.close_connection()
        self.assertEqual(res, None)



import logging

from qdrant_client import QdrantClient

from .credentials import load_qdrant_credentials


class QdrantSingleton:
    __instance = None

    def __init__(self):
        if QdrantSingleton.__instance is not None:
            raise Exception("This class is a singleton!")
        else:
            creds = load_qdrant_credentials()

            # if no api_key was provided
            if creds["api_key"] == "":
                self.client = QdrantClient(
                    host=creds["host"],
                    port=creds["port"],
                )
            else:
                self.client = QdrantClient(
                    host=creds["host"],
                    port=creds["port"],
                    api_key=creds["api_key"],
                    https=False,
                )

            QdrantSingleton.__instance = self

    @staticmethod
    def get_instance():
        if QdrantSingleton.__instance is None:
            QdrantSingleton()
            try:
                _ = QdrantSingleton.__instance.client.get_collections()
                logging.info("QDrant Connected Successfully!")
            except Exception as exp:
                logging.error(f"QDrant not connected! exp: {exp}")

        return QdrantSingleton.__instance

    def get_client(self):
        return self.client



import unittest

from tc_hivemind_backend.db.pg_db_utils import convert_tuple_str


class TestConvertTupleStr(unittest.TestCase):
    def test_convert_tuple_str_single_item(self):
        data = ["item1"]
        result = convert_tuple_str(data)
        self.assertEqual(result, "('item1')")

    def test_convert_tuple_str_multiple_items(self):
        data = ["item1", "item2", "item3"]
        result = convert_tuple_str(data)
        self.assertEqual(result, "('item1', 'item2', 'item3')")

    def test_convert_tuple_str_empty_list(self):
        data = []
        result = convert_tuple_str(data)
        self.assertEqual(result, "()")

    def test_convert_tuple_str_numbers(self):
        data = ["1", "2", "3"]
        result = convert_tuple_str(data)
        self.assertEqual(result, "('1', '2', '3')")

    def test_convert_tuple_str_special_characters(self):
        data = ["@", "#", "$"]
        result = convert_tuple_str(data)
        self.assertEqual(result, "('@', '#', '$')")


if __name__ == "__main__":
    unittest.main()



import logging
from datetime import datetime

import psycopg2
from tc_hivemind_backend.db.postgresql import PostgresSingleton


def setup_db(
    community_id: str, dbname: str, latest_date_query: str | None = None
) -> datetime | None:
    """
    setup the database.
    create a database if not available, else get the latest message saved

    Parameters
    ------------
    community_id : str
        the community id for the case of logging
    dbname : str
        the database name to create or access its database
    guild_id : str
        the guild id to create a database for
    latest_date_query : str | None
        the query to get latest date of a message
        if `None`, then no need to check for latest_message date
        the return would be also `None` if this field was `None`

    Returns
    ---------
    from_date : datetime | None
        in case of no data available it would be None
    """
    msg = f"COMMUNITYID: {community_id} "
    from_date: datetime | None = None
    connection: psycopg2.extensions.connection
    try:
        postgres = PostgresSingleton(dbname=dbname)
        if postgres is not None:
            logging.info(f"{msg}Database {dbname} is already available!")
            postgres.close_connection()
            if latest_date_query is not None:
                logging.info(f"{msg}Checking the latest saved message!")
                from_date = get_latest_msg(community_id, dbname, latest_date_query)
        else:
            logging.warning(
                f"{msg}Database {dbname} is Not available! Creating one instead!"
            )
            # Connecting to default db passed in `.env`
            postgres_default_db = PostgresSingleton(dbname=None)
            connection = postgres_default_db.get_connection()
            connection.autocommit = True
            cursor = connection.cursor()
            logging.info(f"{msg}Creating database {dbname}")
            cursor.execute(f"CREATE DATABASE {dbname};")
            cursor.execute("CREATE EXTENSION IF NOT EXISTS vector;")
            cursor.close()
            connection.close()
    except Exception as exp:
        logging.error(f"{msg}database initialization error: {exp}")

    return from_date


def get_latest_msg(community_id: str, dbname: str, latest_date_query: str):
    from_date: datetime | None = None
    msg = f"COMMUNITYID: {community_id} "

    postgres = PostgresSingleton(dbname=dbname)
    connection = postgres.get_connection()
    connection.autocommit = True
    with connection.cursor() as cursor:
        try:
            # If we had some data previously saved
            # fetch the latest date we wanted to work on it
            logging.info(f"{msg}Loading the latest date from previous data")
            cursor.execute(latest_date_query)
            data = cursor.fetchone()
            if data is not None:
                from_date = data[0]
                logging.info(f"{msg}Latest processed message: {from_date}")
            else:
                logging.info(f"{msg}No processed message, starting from the first!")
        except psycopg2.errors.UndefinedTable:
            logging.warning(f"{msg}No data to get the latest date")

    postgres.close_connection()
    return from_date


def convert_tuple_str(data: list[str]) -> str:
    """
    convert a list of inputs to a string tuple that
    can be queried within database

    Parameters
    ------------
    data : list[str]
        list of string items

    Returns
    ---------
    data_str : str
        the data converted to string tuple
    """
    data_str: str
    if len(data) == 1:
        data_str = f"('{data[0]}')"
    else:
        data_str = str(tuple(data))

    return data_str



import logging

from pymongo import MongoClient

from .credentials import Credentials


class MongoSingleton:
    __instance: "MongoSingleton" | None = None

    def __init__(self):
        if MongoSingleton.__instance is not None:
            raise Exception("This class is a singleton!")
        else:
            connection_uri = get_mongo_uri()
            self.client = MongoClient(connection_uri)
            MongoSingleton.__instance = self

    @staticmethod
    def get_instance():
        if MongoSingleton.__instance is None:
            MongoSingleton()
            try:
                info = MongoSingleton.__instance.client.server_info()
                logging.info(f"MongoDB Connected Successfully! server info: {info}")
            except Exception as exp:
                logging.error(f"MongoDB not connected! exp: {exp}")

        return MongoSingleton.__instance

    def get_client(self):
        return self.client


def get_mongo_uri() -> str:
    mongo_creds = Credentials().load_mongo()
    user = mongo_creds["user"]
    password = mongo_creds["password"]
    host = mongo_creds["host"]
    port = mongo_creds["port"]

    connection = f"mongodb://{user}:{password}@{host}:{port}"

    return connection



import unittest

import psycopg2
from llama_index.core import Document
from llama_index.core.indices.vector_store import VectorStoreIndex
from tc_hivemind_backend.db.credentials import load_postgres_credentials
from tc_hivemind_backend.db.pg_db_utils import setup_db
from tc_hivemind_backend.pg_vector_access import PGVectorAccess


class TestPGVectorAccess(unittest.TestCase):
    def setUpDB(self, dbname: str, table_name: str):
        creds = load_postgres_credentials()
        self.community_id = "123456789a"
        latest_date_query = f"""
            SELECT (metadata_->> 'date')::timestamp AS latest_date
            FROM data_{table_name}
            ORDER BY (metadata_->>'date')::timestamp DESC
            LIMIT 1;
        """
        setup_db(
            community_id=self.community_id,
            dbname=dbname,
            latest_date_query=latest_date_query,
        )
        self.postgres_conn = psycopg2.connect(
            dbname=dbname,
            user=creds["user"],
            password=creds["password"],
            host=creds["host"],
            port=creds["port"],
        )

    def _create_and_save_doc(self, table: str, dbname: str):
        documents = [
            Document(
                text="test_message1",
                metadata={
                    "channel": "channel#1",
                    "author_username": "author#1",
                    "author_global_name": "author_global#1",
                    "date": "2023-08-01",
                },
            )
        ]
        self.setUpDB(dbname=dbname, table_name=table)
        cursor = self.postgres_conn.cursor()
        cursor.execute(f"DROP TABLE IF EXISTS data_{table};")
        self.postgres_conn.commit()
        pg_vector = PGVectorAccess(table_name=table, dbname=dbname, testing=True)

        # Save the documents
        pg_vector.save_documents(self.community_id, documents, verbose=True)
        return documents

    def test_save_documents(self):
        table = "discord"
        guild_id = "1234"
        dbname = f"guild_{guild_id}"

        documents = self._create_and_save_doc(table, dbname)
        cursor = self.postgres_conn.cursor()

        cursor.execute(f"SELECT text, node_id, metadata_ FROM data_{table};")

        data = cursor.fetchall()

        self.assertEqual(len(data), 1)

        # text, node_id, metadata
        text, _, metadata = data[0]

        # nickname was `None`, so it wasn't included in metadata
        self.assertEqual(text, "test_message1")
        self.assertEqual(metadata["channel"], documents[0].metadata["channel"])
        self.assertEqual(
            metadata["author_username"], documents[0].metadata["author_username"]
        )
        self.assertEqual(
            metadata["author_global_name"], documents[0].metadata["author_global_name"]
        )
        self.assertEqual(metadata["date"], documents[0].metadata["date"])
        self.assertNotIn("author_nickname", metadata)

        cursor.close()

    def test_load_index(self):
        table = "discord"
        guild_id = "1234"
        dbname = f"guild_{guild_id}"
        _ = self._create_and_save_doc(table, dbname)

        pg_vector = PGVectorAccess(table_name=table, dbname=dbname, testing=True)
        index = pg_vector.load_index()

        self.assertIsInstance(index, VectorStoreIndex)

        cursor = self.postgres_conn.cursor()
        cursor.close()



import os
import unittest

from tc_hivemind_backend.db.utils.model_hyperparams import load_model_hyperparams


class TestLoadModelHyperparams(unittest.TestCase):
    def setUp(self):
        # Set up environment variables for testing
        os.environ["CHUNK_SIZE"] = "128"
        os.environ["EMBEDDING_DIM"] = "256"

    def tearDown(self):
        # Clean up environment variables after testing
        del os.environ["CHUNK_SIZE"]
        del os.environ["EMBEDDING_DIM"]

    def test_load_model_hyperparams_success(self):
        # Test when environment variables are set correctly
        chunk_size, embedding_dim = load_model_hyperparams()
        self.assertEqual(chunk_size, 128)
        self.assertEqual(embedding_dim, 256)

    def test_load_model_hyperparams_invalid_chunk_size(self):
        # Test when CHUNK_SIZE environment variable is not a valid integer
        os.environ["CHUNK_SIZE"] = "invalid"
        with self.assertRaises(ValueError) as context:
            load_model_hyperparams()
        self.assertEqual(
            str(context.exception), "invalid literal for int() with base 10: 'invalid'"
        )

    def test_load_model_hyperparams_invalid_embedding_dim(self):
        # Test when EMBEDDING_DIM environment variable is not a valid integer
        os.environ["EMBEDDING_DIM"] = "invalid"
        with self.assertRaises(ValueError) as context:
            load_model_hyperparams()
        self.assertEqual(
            str(context.exception), "invalid literal for int() with base 10: 'invalid'"
        )



import unittest

from tc_hivemind_backend.db.credentials import Credentials
from tc_hivemind_backend.db.mongo import get_mongo_uri


class TestCredentialLoadings(unittest.TestCase):
    def setUp(self) -> None:
        self.credentials = Credentials()

    def test_postgres_envs_type(self):
        postgres_creds = self.credentials.load_postgres()

        self.assertIsInstance(postgres_creds, dict)

    def test_postgres_envs_values(self):
        postgres_creds = self.credentials.load_postgres()

        self.assertNotEqual(postgres_creds["host"], None)
        self.assertNotEqual(postgres_creds["password"], None)
        self.assertNotEqual(postgres_creds["user"], None)
        self.assertNotEqual(postgres_creds["port"], None)

        self.assertIsInstance(postgres_creds["host"], str)
        self.assertIsInstance(postgres_creds["password"], str)
        self.assertIsInstance(postgres_creds["user"], str)
        self.assertIsInstance(postgres_creds["port"], str)

    def test_load_qdrant_creds(self):
        qdrant_creds = self.credentials.load_qdrant()

        self.assertIsNotNone(qdrant_creds["host"])
        self.assertIsNotNone(qdrant_creds["port"])
        self.assertIsNotNone(qdrant_creds["api_key"])

    def test_mongo_envs_check_type(self):
        mongo_creds = self.credentials.load_mongo()

        self.assertIsInstance(mongo_creds, dict)

    def test_mongo_envs_values(self):
        mongo_creds = self.credentials.load_mongo()

        self.assertNotEqual(mongo_creds["user"], "")
        self.assertNotEqual(mongo_creds["password"], "")
        self.assertNotEqual(mongo_creds["host"], "")
        self.assertNotEqual(mongo_creds["port"], "")

        self.assertIsInstance(mongo_creds["user"], str)
        self.assertIsInstance(mongo_creds["password"], str)
        self.assertIsInstance(mongo_creds["host"], str)
        self.assertIsInstance(mongo_creds["port"], str)

    def test_redis_envs_check_type(self):
        redis_creds = self.credentials.load_redis()

        self.assertIsInstance(redis_creds, dict)

    def test_redis_envs_values(self):
        redis_creds = self.credentials.load_redis()

        self.assertIsNotNone(redis_creds["password"])
        self.assertIsNotNone(redis_creds["host"])
        self.assertIsNotNone(redis_creds["port"])

        self.assertIsInstance(redis_creds["password"], str)
        self.assertIsInstance(redis_creds["host"], str)
        self.assertIsInstance(redis_creds["port"], str)

    def test_config_mongo_creds(self):
        mongo_uri = get_mongo_uri()

        self.assertIsInstance(mongo_uri, str)
        self.assertIn("mongodb://", mongo_uri)
        self.assertNotIn("None", mongo_uri)
